-- LOOKUP ORDER:
--- user specific permissions
--- user group permissions
--- user implicit level
--- user group implicit level

CREATE TABLE IF NOT EXISTS users (
    id INTEGER PRIMARY KEY,
    uniqueid TEXT NOT NULL UNIQUE,
    implicit_level INTEGER NOT NULL ON CONFLICT REPLACE DEFAULT 0
);

CREATE TABLE IF NOT EXISTS groups (
    id INTEGER PRIMARY KEY,
    name TEXT NOT NULL UNIQUE ON CONFLICT IGNORE, -- Duplicate insert attempt
    implicit_level INTEGER NOT NULL ON CONFLICT REPLACE DEFAULT 0 -- By default has low level
);

CREATE TABLE IF NOT EXISTS group_members (
    user_id INTEGER NOT NULL,
    group_id INTEGER NOT NULL,
    PRIMARY KEY (user_id, group_id)
);

CREATE TABLE IF NOT EXISTS permissions (
    id INTEGER PRIMARY KEY,
    name TEXT NOT NULL UNIQUE ON CONFLICT IGNORE, -- Duplicate insert attempt
    implicit_level INTEGER NOT NULL ON CONFLICT REPLACE DEFAULT 100 -- By default requires high level
);

CREATE TABLE IF NOT EXISTS user_permissions (
    user_id INTEGER NOT NULL,
    permission_id INTEGER NOT NULL,
    permitted BOOLEAN NOT NULL ON CONFLICT REPLACE DEFAULT FALSE,
    PRIMARY KEY (user_id, permission_id)
);

CREATE TABLE IF NOT EXISTS group_permissions (
    -- These are always permitted=True, as there may be more than one per user
    group_id INTEGER NOT NULL,
    permission_id INTEGER NOT NULL,
    PRIMARY KEY (group_id, permission_id)
);

CREATE TRIGGER IF NOT EXISTS user_removed AFTER DELETE ON users
BEGIN
    DELETE FROM group_members WHERE user_id=OLD.id;
    DELETE FROM user_permissions WHERE user_id=OLD.id;
END;

CREATE TRIGGER IF NOT EXISTS group_removed AFTER DELETE ON groups
BEGIN
    DELETE FROM group_members WHERE group_id=OLD.id;
    DELETE FROM group_permissions WHERE group_id=OLD.id;
END;

CREATE TRIGGER IF NOT EXISTS permission_removed AFTER DELETE ON permissions
BEGIN
    DELETE FROM user_permissions WHERE permission_id=OLD.id;
    DELETE FROM group_permissions WHERE permission_id=OLD.id;
END;

-- Insert some default values
INSERT OR IGNORE INTO users (uniqueid, implicit_level) VALUES ('STEAM_ID_LAN', 100);

INSERT OR IGNORE INTO groups (name, implicit_level) VALUES ('user', 0);
INSERT OR IGNORE INTO groups (name, implicit_level) VALUES ('mod', 50);
INSERT OR IGNORE INTO groups (name, implicit_level) VALUES ('admin', 100);